/* 

                          Firewall Builder

                 Copyright (C) 2000 NetCitadel, LLC

  Author:  Vadim Kurland     vadim@vk.crocodile.org

  $Id: Firewall.cpp,v 1.1.2.5 2004/05/10 06:56:12 vkurland Exp $


  This program is free software which we release under the GNU General Public
  License. You may redistribute and/or modify this program under the terms
  of that license as published by the Free Software Foundation; either
  version 2 of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  To get a copy of the GNU General Public License, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include <assert.h>

#include <fwbuilder/libfwbuilder-config.h>

#include <fwbuilder/Firewall.h>

#include <fwbuilder/FWObjectDatabase.h>
#include <fwbuilder/FWObjectReference.h>
#include <fwbuilder/FWOptions.h>
#include <fwbuilder/Interface.h>
#include <fwbuilder/Management.h>

#include <fwbuilder/Policy.h>
#include <fwbuilder/InterfacePolicy.h>
#include <fwbuilder/NAT.h>

#include <fwbuilder/RuleElement.h>

#include <fwbuilder/XMLTools.h>

#include <iostream>

using namespace std;
using namespace libfwbuilder;

const char *Firewall::TYPENAME={"Firewall"};

Firewall::Firewall()
{
    setStr("platform","unknown");
    setStr("host_OS" ,"unknown");
}

Firewall::Firewall(const FWObject *root) : Host(root)
{
    setStr("platform","unknown");
    setStr("host_OS" ,"unknown");

    add( new FirewallOptions() );
    add( new Policy() );
    add( new NAT() );
}


Firewall::~Firewall()  {}

void Firewall::fromXML(xmlNodePtr root) throw(FWException)
{
    const char *n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("platform")));
    assert(n!=NULL);
    setStr("platform", n);
    FREEXMLBUFF(n);

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("version")));
    if (n!=NULL)
    {
        setStr("version", n);
        FREEXMLBUFF(n);
    }

    n=FROMXMLCAST(xmlGetProp(root,TOXMLCAST("host_OS")));
    if (n!=NULL)
    {
        setStr("host_OS", n);
        FREEXMLBUFF(n);
    }

    Host::fromXML(root);
}

xmlNodePtr Firewall::toXML(xmlNodePtr parent) throw(FWException)
{
    xmlNodePtr me = FWObject::toXML(parent, false);
    FWObject *o;
    
    o=getFirstByType( NAT::TYPENAME );
    if (o) o->toXML(me);

    o=getFirstByType( Policy::TYPENAME );
    if (o) o->toXML(me);

    for(FWObjectTypedChildIterator j=findByType(Interface::TYPENAME); j!=j.end(); ++j)
 	if((o=(*j))!=NULL )
 	    o->toXML(me);

    o=getFirstByType( Management::TYPENAME );
    if(o) o->toXML(me);

    o=getFirstByType( FirewallOptions::TYPENAME );
    if (o) o->toXML(me);

    return me;
}

FWOptions* Firewall::getOptionsObject()
{
    return FWOptions::cast( getFirstByType(FirewallOptions::TYPENAME) );
}


Policy* Firewall::getPolicy() 
{ 
    return(Policy::cast(getFirstByType(Policy::TYPENAME)));
}

NAT* Firewall::getNAT()    
{ 
    return(NAT::cast(getFirstByType(NAT::TYPENAME)));
}


bool  Firewall::validateChild(FWObject *o)
{ 
    string otype=o->getTypeName();
    return (FWObject::validateChild(o) && 
	    (otype==Interface::TYPENAME  ||
	     otype==RuleSet::TYPENAME    ||
	     otype==Policy::TYPENAME     ||
	     otype==NAT::TYPENAME        ||
	     otype==Management::TYPENAME ||
	     otype==FirewallOptions::TYPENAME ));
}

void Firewall::replaceRef(FWObject *rs,
                          const string &old_id,const std::string &new_id)
{
    FWReference *ref=FWObjectReference::cast(rs);
    if (ref==NULL)
    {
        for (FWObject::iterator j1=rs->begin(); j1!=rs->end(); ++j1)
            replaceRef( *j1, old_id, new_id);
    } else
    { 
        if (ref->getPointerId()==old_id)
            ref->setPointerId(new_id);
    }
}

FWObject& Firewall::duplicate(const FWObject *obj, bool preserve_id) throw(FWException)
{
    string err="Error creating object with type: ";

//    return FWObject::duplicate(obj,preserve_id);

    checkReadOnly();

    shallowDuplicate(obj,preserve_id);

    destroyChildren();

    Policy   *op =Policy::cast(obj->getFirstByType(Policy::TYPENAME));
    FWObject *pol=addCopyOf(op,preserve_id);
    replaceRef(pol,obj->getId(), getId() );

    NAT      *on =NAT::cast(obj->getFirstByType(NAT::TYPENAME));
    FWObject *nat=addCopyOf(on,preserve_id);
    replaceRef(nat,obj->getId(), getId() );

    FWObjectTypedChildIterator m=obj->findByType(Interface::TYPENAME);
    for ( ; m!=m.end(); ++m ) 
    {
        FWObject *o   =*m;
        FWObject *oa  =o->getFirstByType(IPv4::TYPENAME);
        FWObject *opa =o->getFirstByType(physAddress::TYPENAME);

        FWObject *o1  =addCopyOf(o,preserve_id);
        FWObject *oa1 =o1->getFirstByType(IPv4::TYPENAME);
        FWObject *opa1=o1->getFirstByType(physAddress::TYPENAME);

        replaceRef(pol, o->getId(),   o1->getId()   );
        if (oa!=NULL && oa1!=NULL)   replaceRef(pol, oa->getId(),  oa1->getId() );
        if (opa!=NULL && opa1!=NULL) replaceRef(pol, opa->getId(), opa1->getId());

        replaceRef(nat, o->getId(),   o1->getId()   );
        if (oa!=NULL && oa1!=NULL)   replaceRef(nat, oa->getId(),  oa1->getId() );
        if (opa!=NULL && opa1!=NULL) replaceRef(nat, opa->getId(), opa1->getId());

	RuleSet  *ipolicy=RuleSet::cast(o1->getFirstByType(InterfacePolicy::TYPENAME));
        if (ipolicy)
        {
            replaceRef(ipolicy, obj->getId(), getId()       );
            replaceRef(ipolicy, o->getId(),   o1->getId()   );
            if (oa!=NULL && oa1!=NULL)   replaceRef(ipolicy,oa->getId(),oa1->getId());
            if (opa!=NULL && opa1!=NULL) replaceRef(ipolicy,opa->getId(),opa1->getId());
        }
    }

    FWObject *o=obj->getFirstByType( Management::TYPENAME );
    addCopyOf(o,preserve_id);

    o=obj->getFirstByType( FirewallOptions::TYPENAME );
    addCopyOf(o,preserve_id);

    setDirty(true);

    return *this;
}

